<?php
namespace nur\sery\schema;

use nur\sery\schema\input\Input;
use nur\sery\schema\values\AssocValue;
use nur\sery\schema\values\IValue;
use nur\sery\schema\values\ScalarValue;
use nur\sery\schema\values\SeqValue;
use nulib\StateException;

#XXX
# analyze() et [analyzer_func] --> indique ce qu'est la valeur qu'on vérifixe
#    retourne MISSING (la valeur n'existe pas)
#             NULL (la valeur existe mais est nulle)
#             STRING (la valeur est une chaine qu'il faut parser)
#             INVALID (la valeur n'est pas du bon type et il n'est pas possible de la convertir)
#             VALID (la valeur est dans le bon type)
# extract() et [extractor_func] --> si analyze() retourne STRING, extraire la
#    chaine à parser
# parse() et [parser_func] --> parser la chaine et retourner la valeur
# format() et [formatter_func] --> formatter la valeur et retourner une chaine

class OldSchema {
  /**
   * @var bool true si toutes les clés du schéma doivent exister, avec leur
   * valeur par défaut le cas échéant
   */
  protected $ppEnsureKeys;

  /** @var bool true si les clés doivent être dans l'ordre du schéma */
  protected $ppOrderKeys;

  /**
   * @var bool true si les valeurs doivent être automatiquement vérifiées et
   * corrigées
   */
  protected $ppVerifix;

  function ensureSchema(&$value, $key=null, ?array $params=null): void {
  }

  /** retourner true si la valeur est scalaire */
  function getScalar(&$value, $key=null, ?ScalarValue &$scalar=null): bool {
    if (!$this->isScalar()) return false;
    if ($value instanceof Input) $input = $value;
    else $input = new Input($value);
    $scalar = new ScalarValue($input, $this->schema, $key);
    return true;
  }

  /** retourner true la valeur est un tableau séquentiel */
  function getSeq(&$value, $key=null, ?SeqValue &$seq=null): bool {
    if (!$this->isSeq()) return false;
    if ($value instanceof Input) $input = $value;
    else $input = new Input($value);
    $seq = new SeqValue($input, $this->schema, $key);
    return true;
  }

  /** retourner true la valeur est un tableau associatif */
  function getAssoc(&$value, $key=null, ?AssocValue &$assoc=null): bool {
    if (!$this->isAssoc()) return false;
    if ($value instanceof Input) $input = $value;
    else $input = new Input($value);
    $assoc = new AssocValue($input, $this->schema, $key);
    return true;
  }

  function getValue(&$value, $key=null): IValue {
    if ($this->getScalar($input, $key, $scalar)) return $scalar;
    elseif ($this->getSeq($input, $key, $seq)) return $seq;
    elseif ($this->getAssoc($input, $key, $assoc)) return $assoc;
    else throw StateException::unexpected_state();
  }
}
